from typing import List

DEFAULT_LOG_FILE: str = "virus.log"
DEFAULT_VALIDATION_RULES: str = "src/validations/rules.py"
DEFAULT_DATASET_RULES: str = "src/validations/config.json"
DEFAULT_VALIDATION_COLUMNS: List[str] = [
    "schema_context",
    "column",
    "check",
    "check_number",
    "failure_case",
    "index",
]
# detects: date_time, date, start_date, end_date. not_detected: candidate, candidate_name
DATE_COLUMN_REGEX: str = r"\b(?:\w*_)?date(?:_\w*)?\b"
# detects: year, year_month, year_month_date. not_detected: years, yearmonth
YEAR_COLUMN_REGEX: str = r"\b(?:\w*_)?year(?:_\w*)?\b"
# detects: month, month_name, month_number. not_detected: months, monthname
MONTH_COLUMN_REGEX: str = r"\b(?:\w*_)?month(?:_\w*)?\b"
# detects: year, month, fiscal_year, month_name. not_detected: year_month
DATE_MONTH_COLUMNS_REGEX: str = r"\b(?:\w*_)?(year|month|date)(?:_\w*)?\b"
STATE_COLUMN_REGEX: str = r"\b(?:\w*_)?state(?:_\w*)?\b"
COUNTRY_COLUMN_REGEX: str = r"\b(?:\w*_)?country(?:_\w*)?\b"
FISCAL_YEAR_MONTH_DICT: dict = {
    "April": 1,
    "May": 2,
    "June": 3,
    "July": 4,
    "August": 5,
    "September": 6,
    "October": 7,
    "November": 8,
    "December": 9,
    "January": 10,
    "February": 11,
    "March": 12,
}
YEAR_MONTH_DICT: dict = {
    "January": 1,
    "February": 2,
    "March": 3,
    "April": 4,
    "May": 5,
    "June": 6,
    "July": 7,
    "August": 8,
    "September": 9,
    "October": 10,
    "November": 11,
    "December": 12,
}
SHEET_ID: str = "1RGmfn9_ujELGK03uPeA6hf-JdeIXvn5Ev3aC19abFcw"
STANDARD_NAMES_URL: str = f"https://docs.google.com/spreadsheets/d/{SHEET_ID}/gviz/tq?tqx=out:csv&sheet=standard_names"
COUNTRY_NAMES_URL: str = f"https://docs.google.com/spreadsheets/d/{SHEET_ID}/gviz/tq?tqx=out:csv&sheet=country"
ABBREVIATIONS_URL: str = f"https://docs.google.com/spreadsheets/d/{SHEET_ID}/gviz/tq?tqx=out:csv&sheet=standard_abbrevations"



from datetime import datetime
from typing import List

from pydantic import model_validator
from pydantic_settings import BaseSettings

from .app_logger import log


class DateContinuity(BaseSettings):
    cols: List[str] = []
    format_name: str = "year_month"
    frequency: str = "MS"
    skip_dates: List[str] = []

    @model_validator(mode="after")
    def validate_cols(self):
        format_name = self.format_name
        if format_name not in ["year_month", "fiscal_year_month", "date", "year"]:
            log.error(
                "Invalid format_name, possible values are 'year_month', 'fiscal_year_month', 'date', 'year'"
            )
            raise ValueError("Invalid format_name")
        if format_name in ["date", "year"] and len(self.cols) != 1:
            log.error("date, year format requires exactly 1 column")
            raise ValueError("date, year format requires exactly 1 column")
        elif format_name in ["year_month", "fiscal_year_month"] and len(self.cols) != 2:
            log.error(
                "year_month and fiscal_year_month formats require exactly 2 columns"
            )
            raise ValueError(
                "year_month and fiscal_year_month formats require exactly 2 columns"
            )
        elif self.skip_dates:
            dates = []
            # check format of skip_dates dd-mm-yyyy
            for date in self.skip_dates:
                try:
                    dates.append(datetime.strptime(date, "%d-%m-%Y"))
                except ValueError:
                    log.error("Invalid date format, should be dd-mm-yyyy")
                    raise ValueError("Invalid date format, should be dd-mm-yyyy")
            log.info(f"Skipping dates: {self.skip_dates}")
            self.skip_dates = dates
        return self



import importlib.util
import json
from pathlib import Path
from typing import Union

import pandas as pd
import yaml
from pandera.errors import SchemaErrors

from . import settings
from .app_logger import log
from .assets.constants import FILE_EXTENSIONS

module_name = "rules"
validation_output_columns = settings.DEFAULT_VALIDATION_COLUMNS


class VirusEngine:
    """
    Object facilitate to get the mapping for dataset and schema, and validate the dataset with schema
    """

    def __init__(
        self,
        config_path: str = settings.DEFAULT_DATASET_RULES,
        module_path: str = settings.DEFAULT_VALIDATION_RULES,
    ):
        # Current working directory is always at datasets project dir
        self.cwd = Path.cwd()
        self.schema_config_path = self.cwd / config_path
        self.module_path = module_path

    def _get_schema_mapping(self):
        # There are only 2 file extensions supported: .yaml and .json
        if self.schema_config_path.suffix == ".yaml":
            with open(str(self.schema_config_path), "r") as f:
                schema_mapping = yaml.safe_load(f)
        elif self.schema_config_path.suffix == ".json":
            with open(str(self.schema_config_path), "r") as f:
                schema_mapping = json.load(f)
        else:
            raise ValueError("File format not supported")
        schema_mapping = {
            str(path) if path.endswith(".csv") else f"{path}*.csv": schema
            for path, schema in schema_mapping.items()
        }
        current_dir = Path(".")
        iterated_path_schema_mapping = {
            str(path): schema
            for paths, schema in schema_mapping.items()
            for path in current_dir.glob(paths)
        }
        return iterated_path_schema_mapping

    def retrieve_dataset(self, src: Union[str, Path], **kwargs):
        real_paths = self.cwd.glob(src)
        ## Checks if the generator is empty
        try:
            _ = next(real_paths)
        except StopIteration:
            log.warning(f"Path does not exist: {src}")

        if src.endswith(FILE_EXTENSIONS):
            glob_path = src
        else:
            glob_path = src + "**/*.csv"
        for each_file in (self.cwd).glob(glob_path):
            yield each_file, pd.read_csv(each_file, low_memory=False, **kwargs)

    def validate_with_schema(self, schema_name: str, df: pd.DataFrame) -> pd.DataFrame:
        # create a module spec
        spec = importlib.util.spec_from_file_location(module_name, self.module_path)

        # create a module object
        module = importlib.util.module_from_spec(spec)

        # load the module
        spec.loader.exec_module(module)

        rule_class = getattr(module, schema_name)

        try:
            _ = rule_class.validate(df, lazy=True)
            return pd.DataFrame(columns=validation_output_columns), False
        except SchemaErrors as err:
            return err.failure_cases.drop(columns=["index"]), True



from pathlib import Path
from typing import Union

from pandas import DataFrame
from rich.console import Console
from rich.table import Table

from . import settings


def get_console(
    is_terminal: bool = True,
    log_file_name: str = settings.DEFAULT_LOG_FILE,
):
    # create terminal accordingly
    if not is_terminal:
        console = Console(file=(Path.cwd() / log_file_name).open(mode="w"))
    else:
        console = Console()
    return console


# make a console log function that logs output to a file or terminal based on the config
def console_log(
    console: Console,
    virus_dataframe: DataFrame,
    file_path: Union[str, Path],
):
    # a rule with the file name followed by the rich table for dataframe
    relative_path = str(Path(file_path).relative_to(Path.cwd()))
    console.rule(f"{relative_path}")
    table = Table(show_header=True, header_style="bold")

    # add column names to tables for schema_context,column,check,check_number,failure_case
    # TODO: `overflow` methods seems to have no change over column, thus adding ellipses manually
    table.add_column("schema_context")
    table.add_column("column")
    table.add_column("check")
    table.add_column("check_number")
    table.add_column("failure_case")

    for idx, each_row in enumerate(
        virus_dataframe.to_dict(orient="records"), 1
    ):
        table.add_row(
            str(each_row["schema_context"]),
            str(each_row["column"]),
            str(each_row["check"][:100] + "..."),
            str(each_row["check_number"]),
            str(each_row["failure_case"]),
        )

    console.print(table)



import logging

from rich.logging import RichHandler

# create a logger
logging.basicConfig(
    level="INFO",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)],
)
log = logging.getLogger("virus")



FILE_EXTENSIONS = (".csv", ".parquet")

STATES = {
    "All India",
    "Andaman and Nicobar Islands",
    "Andhra Pradesh",
    "Arunachal Pradesh",
    "Assam",
    "Bihar",
    "Chandigarh",
    "Chhattisgarh",
    "Dadra and Nagar Haveli",
    "Daman and Diu",
    "Delhi",
    "Goa",
    "Gujarat",
    "Haryana",
    "Himachal Pradesh",
    "Jammu and Kashmir",
    "Jharkhand",
    "Karnataka",
    "Kerala",
    "Ladakh",
    "Lakshadweep",
    "Madhya Pradesh",
    "Maharashtra",
    "Manipur",
    "Meghalaya",
    "Mizoram",
    "Nagaland",
    "Odisha",
    "Others",
    "Puducherry",
    "Punjab",
    "Rajasthan",
    "Sikkim",
    "Tamil Nadu",
    "Telangana",
    "Dadra and Nagar Haveli and Daman and Diu",
    "Tripura",
    "Uttar Pradesh",
    "Uttarakhand",
    "West Bengal",
}



import calendar
import re

import pandas as pd

from . import settings
from .app_logger import log


def check_dfcols(df: pd.core.frame.DataFrame, cols: list) -> bool:
    """
    Check if the columns are present in the dataframe
    """
    if not set(cols).issubset(df.columns):
        log.error(f"Columns {cols} not found in the DataFrame")
        return True
    else:
        return False


def get_date_month_cols(df: pd.core.frame.DataFrame) -> list:
    return df.columns[
        df.columns.str.extract(settings.DATE_MONTH_COLUMNS_REGEX, expand=False).notna()
    ].tolist()


def convert_month_values(df, sorting_cols):
    cols = [
        value for value in sorting_cols if re.search(settings.MONTH_COLUMN_REGEX, value)
    ]

    if "fiscal_year" in df.columns:
        map_values = settings.FISCAL_YEAR_MONTH_DICT
    else:
        map_values = settings.YEAR_MONTH_DICT
    for col in cols:
        df[col] = df[col].map(map_values)

    return df


def convert_date_col_format(data, sorting_cols):
    cols = [col for col in sorting_cols if re.search(settings.DATE_COLUMN_REGEX, col)]

    for col in cols:
        try:
            data[col] = pd.to_datetime(data[col], format="%d-%m-%Y")
        except Exception as e:
            raise e
    return data


def year_month_continuity_check(df, year_col, month_col, skip_dates):
    data = df[[year_col, month_col]].drop_duplicates().copy()
    try:
        dates = pd.to_datetime(
            data[year_col].astype(str) + "-" + data[month_col].astype(str),
            format="%Y-%B",
        )
    except ValueError as e:
        log.error(
            f"Check cols order it should be 'year, month' and respective values: {e}"
        )
        return False
    return date_continuity_check(dates, skip_dates=skip_dates)


def date_col_continuity_check(df, date_col, freq, skip_dates):
    data = df[date_col].drop_duplicates().copy()
    try:
        dates = pd.to_datetime(data, format="%d-%m-%Y")
    except ValueError as e:
        log.error(
            f"Check '{date_col}' column values they should be in 'dd-mm-yyyy' format: {e}"
        )
        return False
    return date_continuity_check(dates, freq=freq, skip_dates=skip_dates)


def year_col_continuity_check(df, year_col, skip_dates):
    data = df[year_col].drop_duplicates().copy()
    try:
        dates = pd.to_datetime(data, format="%Y")
    except ValueError as e:
        log.error(
            f"Check '{year_col}' column values they should be in 'yyyy' format: {e}"
        )
        return False
    return date_continuity_check(dates, freq="YS", skip_dates=skip_dates)


def fiscal_year_month_continuity_check(df, fiscal_year_col, month_col, skip_dates):
    data = df[[fiscal_year_col, month_col]].drop_duplicates().copy()
    month_map = {month: idx for idx, month in enumerate(calendar.month_name[1:], 1)}
    data["month_num"] = data[month_col].map(month_map)
    data["year"] = data.apply(
        lambda x: int(x[fiscal_year_col].split("-")[0])
        if int(x["month_num"]) >= 4
        else int(x[fiscal_year_col].split("-")[0]) + 1,
        axis=1,
    )
    data["date"] = pd.to_datetime(
        dict(year=data["year"], month=data["month_num"], day=1)
    )
    try:
        dates = pd.to_datetime(data["date"], format="%d-%m-%Y")
    except ValueError as e:
        log.error(
            f"Check cols order it should be 'fiscal_year, month' and respective values: {e}"
        )
        return False
    return date_continuity_check(dates, skip_dates=skip_dates)


def date_continuity_check(dates, freq="MS", skip_dates=None):
    min_date = dates.min()
    max_date = dates.max()
    intermediate_dates = pd.date_range(min_date, max_date, freq=freq)
    missing_dates = set(intermediate_dates) - set(dates)
    if skip_dates:
        missing_dates = missing_dates - set(skip_dates)
    if missing_dates:
        log.error(f"Missing dates in data: {sorted(missing_dates)}")
        return False
    return True



import calendar
import re

import pandera as pa
from pandera.api.pandas.model import DataFrameModel

from . import settings
from .app_logger import log
from .assets.constants import STATES
from .assets.names import factly_std_names
from .models import DateContinuity
from .utils import (
    check_dfcols,
    convert_date_col_format,
    convert_month_values,
    date_col_continuity_check,
    fiscal_year_month_continuity_check,
    get_date_month_cols,
    year_col_continuity_check,
    year_month_continuity_check,
)


class BaseSchemaModel(DataFrameModel):
    class Config:
        strict = True
        ordered = True
        coerce = True


class FactlyDatasetSchema(BaseSchemaModel):
    _duplicate_cols = None
    _sorting_cols = None

    _check_duplicates = True
    _save_duplicates = False
    _check_sort = True
    _check_column_names = True
    _check_white_space = True
    _check_state_names = True
    _check_month_names = True
    _check_country_names = True
    _check_null_values_in_date_cols = True
    _check_date_continuity = False
    _date_continuity_cols = {}

    # Cache for validation results
    _month_values_check_result = False
    _null_values_in_date_cols_check_result = False

    @pa.dataframe_check
    def check_column_names(cls, df):
        """
        Check for special characters and capital letters in all column names at once
        """
        if not cls._check_column_names:
            return True
        return df.columns.str.contains(r"^[a-z]+(?:_[a-z]+)*$", regex=True).all()

    @pa.dataframe_check
    def white_space_check(cls, df):
        """
        Check for multiple_white_spaces in middle and leading or tailing spaces in the dataframe
        """
        if not cls._check_white_space:
            return True
        return (
            ~df.map(
                lambda x: isinstance(x, str) and bool(re.search(r"\s{2,}|^\s|\s$", x))
            )
            .any()
            .any()
        )

    @pa.dataframe_check
    def state_names_check(cls, df):
        """
        Check for state names in the dataframe
        """
        if not cls._check_state_names:
            return True

        state_cols = df.columns[
            df.columns.str.contains(settings.STATE_COLUMN_REGEX, regex=True)
        ].values.tolist()
        for state_col in state_cols:
            if not df[state_col].isin(STATES).all():
                return False
        return True

    @pa.dataframe_check
    def country_names_check(cls, df):
        """
        Check for country names in the dataframe
        """
        if not cls._check_country_names:
            return True

        country_cols = df.columns[
            df.columns.str.contains(settings.COUNTRY_COLUMN_REGEX, regex=True)
        ].values.tolist()
        if country_cols:
            country_names = factly_std_names("country")
            for country_col in country_cols:
                if not df[country_col].isin(country_names).all():
                    return False
        return True

    @pa.dataframe_check
    def check_null_values_in_temporal(cls, df):
        """
        Check for null values in year or month columns
        """
        if not cls._check_null_values_in_date_cols:
            return True
        cols = get_date_month_cols(df)
        if cols and df.loc[:, cols].isnull().any().any():
            return False
        cls._null_values_in_date_cols_check_result = True
        return True

    @pa.dataframe_check
    def month_values_check(cls, df):
        """
        Check for month values in the dataframe
        """
        if not cls._check_month_names:
            return True
        month_cols = df.columns[
            df.columns.str.contains(settings.MONTH_COLUMN_REGEX, regex=True)
        ].values.tolist()
        for month_col in month_cols:
            if (
                not df[month_col]
                .isin([month for month in calendar.month_name[1:]])
                .all()
            ):
                return False
        cls._month_values_check_result = True
        return True

    @pa.dataframe_check
    def check_date_continuity(cls, df):
        """
        Check for continuity in date columns
        """
        if not cls._check_date_continuity:
            return True

        if not cls._check_month_names:
            log.warning("Cannot check date continuity without month values check")
            return False
        if not cls._null_values_in_date_cols_check_result:
            log.warning(
                "Skipping date continuity check because null values in date cols check failed"
            )
            return False

        # Skip date continuity check if month values check fails
        if not cls._month_values_check_result:
            log.warning(
                "Skipping date continuity check because month values check failed"
            )
            return False
        if not cls._date_continuity_cols:
            if "year" in df.columns and "month" in df.columns:
                cls._date_continuity_cols = {
                    "cols": ["year", "month"],
                    "format_name": "year_month",
                }
            elif "year" in df.columns:
                cls._date_continuity_cols = {
                    "cols": ["year"],
                    "format_name": "year",
                }
            elif "fiscal_year" in df.columns and "month" in df.columns:
                cls._date_continuity_cols = {
                    "cols": ["fiscal_year", "month"],
                    "format_name": "fiscal_year_month",
                }
            elif "date" in df.columns:
                cls._date_continuity_cols = {
                    "cols": ["date"],
                    "format_name": "date",
                    "frequency": "D",
                }
            else:
                log.error("Date columns not found")
                return False

        cls._date_continuity_cols = DateContinuity(**cls._date_continuity_cols)
        log.info(f"Date Continuity Config: {cls._date_continuity_cols}")

        if cls._date_continuity_cols.format_name == "year_month":
            return year_month_continuity_check(
                df,
                cls._date_continuity_cols.cols[0],
                cls._date_continuity_cols.cols[1],
                cls._date_continuity_cols.skip_dates,
            )
        elif cls._date_continuity_cols.format_name == "fiscal_year_month":
            return fiscal_year_month_continuity_check(
                df,
                cls._date_continuity_cols.cols[0],
                cls._date_continuity_cols.cols[1],
                cls._date_continuity_cols.skip_dates,
            )
        elif cls._date_continuity_cols.format_name == "date":
            return date_col_continuity_check(
                df,
                cls._date_continuity_cols.cols[0],
                cls._date_continuity_cols.frequency,
                cls._date_continuity_cols.skip_dates,
            )
        elif cls._date_continuity_cols.format_name == "year":
            return year_col_continuity_check(
                df,
                cls._date_continuity_cols.cols[0],
                cls._date_continuity_cols.skip_dates,
            )
        else:
            log.error("Invalid _date_continuity_cols")
            return False

    @pa.dataframe_check
    def check_duplicates(cls, df):
        """
        Check for duplicates in the dataframe
        """
        if not cls._check_duplicates:
            return True
        if cls._duplicate_cols is not None and check_dfcols(df, cls._duplicate_cols):
            return False

        cls._duplicate_cols = (
            df.columns.tolist() if cls._duplicate_cols is None else cls._duplicate_cols
        )
        log.info(f"Checking for Duplicates on columns: {cls._duplicate_cols}")
        if cls._save_duplicates:
            cls._duplicates = df[
                df.duplicated(subset=cls._duplicate_cols, keep=False)
            ].copy()
            cls._duplicates.sort_values(
                by=cls._duplicate_cols, ascending=False, inplace=True
            )
            cls._duplicates.to_csv("duplicates.log", index=False)

        if df.duplicated(subset=cls._duplicate_cols).any():
            log.error(
                "Duplicates found in the dataframe, to check duplicates run with _save_duplicates=True"
            )
            return False

        return True

    @pa.dataframe_check
    def check_sorting(cls, df):
        """
        Check for sorting in the dataframe
        """
        if not cls._check_sort:
            return True
        if not cls._check_month_names:
            log.warning("Cannot check sorting without month values check")
            return False
        if not cls._null_values_in_date_cols_check_result:
            log.warning(
                "Skipping sorting check because null values in date cols check failed"
            )
            return False
        if not cls._month_values_check_result:
            log.warning("Skipping sorting check because month values check failed")
            return False
        if cls._sorting_cols is None:
            cls._sorting_cols = get_date_month_cols(df)
        elif check_dfcols(df, cls._sorting_cols):
            return False

        if not cls.check_null_values_in_temporal(df):
            log.error(f"Null values found in columns {cls._sorting_cols}")
            return False

        data = convert_date_col_format(df, cls._sorting_cols)
        data = df.copy()
        data = convert_month_values(data, cls._sorting_cols)

        return data[cls._sorting_cols].equals(
            data[cls._sorting_cols].sort_values(
                by=cls._sorting_cols, kind="mergesort", ascending=False
            )
        )



import pandas as pd

from .. import settings


def factly_std_names(filter: str = None):
    if filter is None or filter == "":
        raise Exception("filter is required")

    if filter == "country":
        data = pd.read_csv(settings.COUNTRY_NAMES_URL)
        return data["standard_country_name"].drop_duplicates().dropna().tolist()
    else:
        data = pd.read_csv(settings.STANDARD_NAMES_URL)
        if filter not in data.columns:
            raise Exception("Invalid filter")
        return data[filter].dropna().tolist()



import json
import sys
from pathlib import Path
from typing import List

import typer
from rich.console import Console
from rich.table import Table

from . import settings
from .app_logger import log
from .engine import VirusEngine
from .textwrapper import console_log, get_console


def check_file_exist(path):
    cwd = Path.cwd()
    if not (cwd / path).exists():
        raise typer.BadParameter(f"File does not exist : {path}")
    return path


def check_files_exist(paths):
    cwd = Path.cwd()

    for path in paths:
        real_paths = cwd.glob(path)
        ## Checks if the generator is empty
        try:
            _ = next(real_paths)
        except StopIteration:
            raise typer.BadParameter(f"File does not exist : {path}")
    return paths


app = typer.Typer(
    name="virus",
    add_completion=False,
    help="Virus (Validation is Required for Understanding Schema)",
)


@app.command("validate")
def validate(
    src: List[str] = typer.Argument(
        ..., help="Path to the source file", callback=check_files_exist
    ),
    config_path: str = typer.Option(
        settings.DEFAULT_DATASET_RULES,
        help="Path to the config file with Dataset and rules mapping",
        callback=check_file_exist,
    ),
    is_terminal: bool = typer.Option(True, help="Output to terminal"),
    log_file_path: str = typer.Option(
        settings.DEFAULT_LOG_FILE, help="Path to the log file"
    ),
    rules_path: str = typer.Option(
        settings.DEFAULT_VALIDATION_RULES,
        help="Path to the rules file e.g. src/rules.py",
        callback=check_file_exist,
    ),
):
    cwd = Path(".")
    console_out = get_console(is_terminal, log_file_path)
    virus = VirusEngine(config_path, rules_path)
    # add extension .csv if not present in src
    src = [str(path) if path.endswith(".csv") else f"{path}*.csv" for path in src]
    schema_mapping = virus._get_schema_mapping()
    # get all files from the src
    files = set()
    is_error = False
    for dir_path in src:
        files.update(str(file_path) for file_path in cwd.glob(dir_path))
    if files:
        for each_path in files:
            dataset_schema = schema_mapping.get(each_path, False)
            if dataset_schema:
                for each_file, each_df in virus.retrieve_dataset(each_path):
                    response, error = virus.validate_with_schema(
                        dataset_schema, each_df
                    )
                    if error:
                        is_error = True
                    console_log(console_out, response, each_file)
            else:
                log.error(f"Schema not found for {each_path}")
                is_error = True
    else:
        log.error("No files found in the given path")
        is_error = True
    return sys.exit(1) if is_error else sys.exit(0)


@app.command("validate-all")
def validate_all(
    config_path: str = typer.Option(
        settings.DEFAULT_DATASET_RULES,
        help="Path to the config file with Dataset and rules mapping",
        callback=check_file_exist,
    ),
    is_terminal: bool = typer.Option(True, help="Output to terminal"),
    log_file_path: str = typer.Option(
        settings.DEFAULT_LOG_FILE, help="Path to the log file"
    ),
    rules_path: str = typer.Option(
        settings.DEFAULT_VALIDATION_RULES,
        help="Path to the rules file e.g. src/rules.py",
        callback=check_file_exist,
    ),
):
    console_out = get_console(is_terminal, log_file_path)
    virus = VirusEngine(config_path, rules_path)
    schema_mapping = virus._get_schema_mapping()
    is_error = False

    # if schema_mapping is empty, log error and exit
    if schema_mapping:
        for each_path, dataset_schema in schema_mapping.items():
            if dataset_schema:
                for each_file, each_df in virus.retrieve_dataset(each_path):
                    response, error = virus.validate_with_schema(
                        dataset_schema, each_df
                    )
                    if error:
                        is_error = True
                    console_log(console_out, response, each_file)
            else:
                log.error(f"Schema not found for {each_path}")
                is_error = True
    else:
        log.error("Schema not found in the config file")
        is_error = True
    return sys.exit(1) if is_error else sys.exit(0)


@app.command("list")
def list_schema(
    schema_path: Path = typer.Option(
        (Path.cwd() / settings.DEFAULT_DATASET_RULES),
        help="Path to the schema file",
        callback=check_file_exist,
    )
):
    config_dict = json.loads(schema_path.read_text(encoding="UTF-8"))
    console = Console()
    table = Table(show_header=True, header_style="bold", title="Schema List")
    table.add_column("Dataset Path", justify="left", style="cyan", no_wrap=True)
    table.add_column("Schema Name", justify="left", style="magenta")

    for each_path, each_schema in config_dict.items():
        table.add_row(each_path, each_schema)
    console.print(table)


def main():
    app()
